unit unitmain;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, Unix, FileUtil, TAGraph, TASeries, Forms, Controls, Graphics,
  Dialogs, ExtCtrls, StdCtrls, ComCtrls, unitlib;

Const RET_OK    = 0;

      _128 = 128;
      _256 = _128 * 2;
      _512 = _256 * 2;
      _1K = _512 * 2;
      _2K = _1K * 2;
      _4K = _2K * 2;
      _10K = _2K * 5;
      _100K = _10K * 10;
      _500k = _100K * 5;
      _1M = _500K * 2;
      _2M = _1M * 2;
      _3M = _1M * 3;
      _4M = _1M * 4;
      _5M = _1M * 5;
      _10M = _5M * 2;
      _20M = _10M * 2;
      _50M = _10M * 5;
      _100M = _10M * 10;
      _1G = _100M * 10;

type

  { TThCamReader }

  TThCamReader = class(TThread)
    private
      FCamBuf : Array of Word;
      FSize   : DWORD;
      FBytesRead : Integer;
      FBytesAvailable : Integer;
      FFPS    : dword;
      FNumFrames : dword;
      procedure ShowStatus;
      Function GetUSec : UInt64;
    protected
      procedure Execute; override;
    public
      Constructor Create(CreateSuspended : boolean);
      destructor  Destroy();override;
      property BufSize : DWORD read FSize write FSize;
    end;

  { TForm1 }

  TForm1 = class(TForm)
    btn_enum: TButton;
    btn_exptime: TButton;
    btn_open: TButton;
    btn_i2cwrite: TButton;
    btn_i2cread: TButton;
    btn_geti2cstat: TButton;
    btn_savesettings: TButton;
    btn_setscl: TButton;
    btn_getscl: TButton;
    btn_clear: TButton;
    btn_start: TButton;
    btn_info: TButton;
    cb_devices: TComboBox;
    cb_fullscale: TComboBox;
    cb_i2cfreq: TComboBox;
    cb_mode: TComboBox;
    Chart1: TChart;
    Chart1LineSeries1: TLineSeries;
    ed_exptime: TEdit;
    ed_slvaddr: TEdit;
    ed_i2ccnt: TEdit;
    ed_i2cdata: TEdit;
    Label11: TLabel;
    Label13: TLabel;
    Label14: TLabel;
    Label15: TLabel;
    Label16: TLabel;
    Label17: TLabel;
    lb_gain: TLabel;
    lb_i2cstat: TLabel;
    i2cmemo: TMemo;
    lb_offset: TLabel;
    PageControl1: TPageControl;
    Panel1: TPanel;
    Panel2: TPanel;
    StatusBar1: TStatusBar;
    TabSheet1: TTabSheet;
    TabSheet5: TTabSheet;
    tb_gain: TTrackBar;
    tb_offset: TTrackBar;
    procedure btn_clearClick(Sender: TObject);
    procedure btn_enumClick(Sender: TObject);
    procedure btn_exptimeClick(Sender: TObject);
    procedure btn_geti2cstatClick(Sender: TObject);
    procedure btn_getsclClick(Sender: TObject);
    procedure btn_i2creadClick(Sender: TObject);
    procedure btn_i2cwriteClick(Sender: TObject);
    procedure btn_infoClick(Sender: TObject);
    procedure btn_savesettingsClick(Sender: TObject);
    procedure btn_setsclClick(Sender: TObject);
    procedure btn_startClick(Sender: TObject);
    procedure cb_devicesChange(Sender: TObject);
    procedure cb_fullscaleChange(Sender: TObject);
    procedure cb_modeChange(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure tb_gainChange(Sender: TObject);
    procedure tb_offsetChange(Sender: TObject);
  private
    { private declarations }
  public
    { public declarations }
    procedure to_statusbar(stxt : string);
    procedure open_device;
    procedure readsettings;
    procedure StartCamThread(ABufSize : DWORD);
    procedure StopCamThread;
    function FormatSize(ASize : Integer) : String;
  end;

var
  Form1        : TForm1;
  CamThread    : TThCamReader;
  PipeSize     : Integer;
  PacketLength : Integer;
  wSensorType  : WORD;
  wPixelCount  : WORD;


implementation
uses unitinfo;

{$R *.lfm}

{ TThCamReader }

procedure TThCamReader.ShowStatus;
var i : integer;
begin
  form1.Chart1LineSeries1.BeginUpdate;
  for i := 0 to (FBytesRead DIV 2) - 1 do
    form1.Chart1LineSeries1.SetYValue(i,FCamBuf[i]);
  form1.Chart1LineSeries1.EndUpdate;

  Form1.Statusbar1.Panels[1].Text := 'FPS: '+IntToStr(FFPS)+' ('+IntToStr(FNumFrames)+')';
  Form1.Statusbar1.Panels[2].Text := 'MEM: '+Form1.FormatSize(FBytesAvailable);
  if form1.cb_mode.ItemIndex = 0 then form1.btn_start.Caption := 'Start';
end;

function TThCamReader.GetUSec: UInt64;
Var TZ : TimeVal;
Begin
  fpGetTimeOfDay(@TZ,Nil);
  Result := TZ.tv_usec + TZ.tv_sec*1000000;
End;

procedure TThCamReader.Execute;
var mSecStart, SecStart : UInt64;
    tmp_frames : dword;
begin
  FNumFrames := 0;
  tmp_frames := 0;
  SetLength(FCamBuf,FSize DIV 2);
  SecStart := GetUSec;
  mSecStart := GetUSec;
  while (not Terminated) do
  begin
    FFPS := (ls_getfps DIV FSize);
    FBytesRead := ls_getpipe(@(FCamBuf[0]),FSize);
    if FBytesRead > 0 then
    begin
      Inc(tmp_frames);
      if (GetUSec - SecStart) >= 1000000 then //1s
      begin
        SecStart := GetUSec;
        FNumFrames := tmp_frames;
        tmp_frames := 0;
      end;
      if (GetUSec - mSecStart) >= 200000 then //TAChart component is too slow
      begin                                   //update chart & info every 200ms
        mSecStart := GetUSec;
        FBytesAvailable := ls_getpipe(nil,0);
        Synchronize(@ShowStatus);
      end;
    end else ls_waitforpipe(100);
  end;
end;

constructor TThCamReader.Create(CreateSuspended: boolean);
begin
  FSize := _4K;
  inherited Create(CreateSuspended);
end;

destructor TThCamReader.Destroy;
begin
  SetLength(FCamBuf,0);
  inherited Destroy;
end;

{ TForm1 }

procedure TForm1.FormCreate(Sender: TObject);
var i : integer;
begin
  PipeSize     := _1M;
  wPixelCount  := _1K; //_2K;
  PacketLength := wPixelCount * 2;
  wSensorType  := 0;

  ls_initialize(PipeSize,PacketLength);
  CamThread := nil;
  for i := 0 to wPixelCount-1 do
    Chart1LineSeries1.AddXY(i,1);
end;

procedure TForm1.to_statusbar(stxt: string);
begin
  StatusBar1.Panels[0].Text := stxt;
end;

procedure TForm1.StartCamThread(ABufSize : DWORD);
begin
  if CamThread = nil then
  begin
    CamThread := TThCamReader.Create(true);
    CamThread.BufSize := ABufSize;
    CamThread.Start;
  end;
end;

procedure TForm1.StopCamThread;
begin
  if CamThread <> nil then
  begin
    CamThread.Terminate;
    CamThread.WaitFor;
    FreeAndNil(CamThread);
  end;
end;

function TForm1.FormatSize(ASize: Integer): String;
begin
  if ASize > _1G then
  begin
    result := Format('%f',[ASize/_1G]) +' GB';
  end else if ASize > _1M then
  begin
    result := Format('%f',[ASize/_1M]) +' MB';
  end else if ASize > _1K then
  begin
    result := Format('%f',[ASize/_1K]) +' KB';
  end else result := IntToStr(ASize) +' Bytes';
end;

procedure TForm1.open_device;
var ierr, i : integer;
    tmp_packetlength : Integer = 0;
    tmp_pixel : word = 0;
    tmp_sensor : word = 0;
begin
  if cb_devices.Items.Count = 0 then exit;

  if ls_currentdeviceindex > -1 then
  begin
    StopCamThread;
    ls_closedevice;
    btn_open.Caption := 'Open';
    to_statusbar('USB DEVICE CLOSED');
  end else
  begin
    ierr := ls_opendevicebyindex(cb_devices.ItemIndex);
    if  ierr = RET_OK then
    begin
      ierr := ls_getpacketlength(tmp_packetlength,1000);
      if ierr = RET_OK then
      begin
        if tmp_packetlength <> PacketLength then
        begin
          PacketLength := tmp_packetlength;
          ls_closedevice;
          ls_setpacketlength(PacketLength);
          ierr := ls_opendevicebyindex(cb_devices.ItemIndex);
          if  ierr <> RET_OK then
          begin
            to_statusbar('OPEN DEVICE: '+ls_geterrorstring(ierr));
            exit;
          end;
        end;
      end else
      begin
        to_statusbar('GET PACKELENGTH: '+ls_geterrorstring(ierr));
        ShowMessage('Unable to read Packetlength; value of ('+IntToStr(PacketLength)+') Bytes will be used!');
      end;

      ierr := ls_getsensortype(tmp_sensor,tmp_pixel,1000);
      if ierr = RET_OK then
      begin
        wSensorType := tmp_sensor;
        if wPixelCount <> tmp_pixel then
        begin
          wPixelCount := tmp_pixel;
          Chart1LineSeries1.Clear;
          for i := 0 to wPixelCount-1 do
             Chart1LineSeries1.AddXY(i,1);
        end;
      end else
      begin
        to_statusbar('GET SENSORTYPE: '+ls_geterrorstring(ierr));
        ShowMessage('Unable to read PixelCount; Value of ('+IntToStr(wPixelCount)+') Pixel will be used!');
      end;

      readsettings;
      StartCamThread(PacketLength);
      btn_open.Caption := 'Close';
    end;
    to_statusbar('OPEN DEVICE: '+ls_geterrorstring(ierr));
  end;
end;

procedure TForm1.btn_enumClick(Sender: TObject);
var cnt, i : integer;
begin
  cb_devices.Clear;
  StopCamThread;
  btn_open.Caption := 'Open';
  cnt := ls_enumdevices;
  if cnt > 0 then
  begin
    to_statusbar(IntToStr(cnt)+' DEVICE(S) FOUND');
    for i := 0 to cnt -1 do
    begin
      cb_devices.Items.Add(ls_getproductname(i)+' '+ls_getserialnumber(i));
    end;
    cb_devices.ItemIndex:= 0;
    open_device;
  end else to_statusbar('No DEVICES FOUND');
end;

procedure TForm1.cb_devicesChange(Sender: TObject);
begin
  open_device;
end;

procedure TForm1.btn_startClick(Sender: TObject);
var ierr : integer;
    ucstate : byte = 0;
begin
  ierr := ls_getstate(ucstate,1000);
  if ierr <> 0 then
  begin
    to_statusbar('GET STATE: '+ls_geterrorstring(ierr));
    exit;
  end;
  ucstate := (ucstate xor 1) and 1;
  ierr := ls_setstate(ucstate,1000);
  if ierr = 0 then
  begin
    if ucstate = 1 then btn_start.caption := 'Stop'
    else btn_start.caption := 'Start';
  end else to_statusbar('SET STATE: '+ls_geterrorstring(ierr));
end;

procedure TForm1.btn_exptimeClick(Sender: TObject);
var ierr : integer;
    dwexptime : DWORD;
begin
  dwexptime := StrToIntDef(ed_exptime.Text,10000);
  ed_exptime.Text := IntToStr(dwexptime);
  ierr := ls_setinttime(dwexptime,1000);
  to_statusbar('SET INT TIME: '+ls_geterrorstring(ierr));

  ierr := ls_getinttime(dwexptime,1000);
  if ierr = RET_OK then
    ed_exptime.Text := IntToStr(dwexptime);
end;

procedure TForm1.cb_fullscaleChange(Sender: TObject);
var ierr : integer;
begin
  ierr := ls_setadcconfig(cb_fullscale.ItemIndex shl 7,1000);
  to_statusbar('SET FULL SCALE: '+ls_geterrorstring(ierr));
end;

procedure TForm1.cb_modeChange(Sender: TObject);
var ierr : integer;
begin
  ierr := ls_setmode(cb_mode.ItemIndex,1000);
  if ierr = RET_OK then
  begin
    if cb_mode.ItemIndex = 0 then
      btn_start.Caption := 'Start';
  end;
  to_statusbar('SET MODE: '+ls_geterrorstring(ierr));
end;

procedure TForm1.tb_gainChange(Sender: TObject);
var ierr : integer;
begin
  ierr := ls_setadcpga1(tb_gain.Position,1000);
  lb_gain.caption := Format('%.4f V/V',[6 / (1+(5*((63-tb_gain.Position)/63)))]);
  to_statusbar('SET GAIN: '+ls_geterrorstring(ierr));
end;

procedure TForm1.tb_offsetChange(Sender: TObject);
var fNewValue : Single;
    wValue : word;
    Sign   : word;
    ierr  : integer;
begin
  fNewValue := tb_offset.Position * 1.2;
  if (fNewValue < -300) then fNewValue := -300;
  if (fNewValue > 300) then fNewValue := 300;

  wvalue := abs(tb_offset.Position);
  if tb_offset.Position < 0 then sign := $100
  else sign := 0;
  wvalue := wvalue or sign;
  ierr := ls_setadcoffset1(wvalue,1000);

  lb_offset.caption := Format('%.1f mV',[fNewValue]);
  to_statusbar('SET OFFSET: '+ls_geterrorstring(ierr));
end;

procedure TForm1.readsettings;
Var ierr : integer;
    wValue : WORD = 0;
    fValue : Single;
    iValue : Integer;
    dwValue : DWORD = 0;
    ucValue : Byte = 0;
begin
  ierr := ls_getadcconfig(wValue,1000);
  if ierr = RET_OK then
  begin
    cb_fullscale.OnChange := nil;
    cb_fullscale.ItemIndex := (wvalue shr 7) and $01;
    cb_fullscale.OnChange := @cb_fullscaleChange;
  end;

  ierr := ls_getadcpga1(wValue,1000);
  if ierr = RET_OK then
  begin
    tb_gain.OnChange := nil;
    fValue := 6 / (1+(5*((63-wValue)/63)));
    if fValue < 1 then fValue := 1;
    if fValue > 6 then fValue := 6;
    tb_gain.Position  := wValue;
    lb_gain.caption := Format('%.4f V/V',[fValue]);
    tb_gain.OnChange := @tb_gainChange;
  end;

  ierr := ls_getadcoffset1(wValue,1000);
  if ierr = RET_OK then
  begin
    tb_offset.OnChange := nil;

    ivalue := wvalue and $FF;
    if ((wvalue and $100) = $100) then ivalue := ivalue * (-1);

    fvalue := (ivalue * 1.2);
    if fvalue < -300 then fvalue := -300;
    if fvalue > 300 then fvalue := 300;
    tb_offset.position := ivalue;
    lb_offset.caption := Format('%.1f mV',[fValue]);
    tb_offset.OnChange := @tb_offsetChange;
  end;

  ierr := ls_getinttime(dwvalue,1000);
  if ierr = RET_OK then
    ed_exptime.Text := IntToStr(dwvalue);

  ierr := ls_getstate(ucValue,1000);
  if ierr = RET_OK then
  begin
    if ucValue = 0 then btn_start.caption := 'Start'
    else btn_start.caption := 'Stop';
  end;

  ierr := ls_getmode(ucValue,1000);
  if ierr = RET_OK then
  begin
    cb_mode.OnChange := nil;
    cb_mode.ItemIndex := ucValue and $03;
    cb_mode.OnChange := @cb_modeChange;
  end;
end;

procedure TForm1.btn_savesettingsClick(Sender: TObject);
var ierr : integer;
begin
  ierr := ls_savesettings(1000);
  to_statusbar('SAVE SETTINGS: '+ls_geterrorstring(ierr));
end;

procedure TForm1.btn_clearClick(Sender: TObject);
begin
  i2cmemo.Clear;
end;

procedure TForm1.btn_i2creadClick(Sender: TObject);
var buf : array[0..63] of byte;
    pBuf : Pointer;
    wLength, i : WORD;
    ierr : integer;
    stri : string;
    adr : byte;
    ucstate : byte = 0;
begin
  pBuf := @buf[0];
  wLength := strtoint(ed_i2ccnt.text);
  adr := strtoint('$'+ed_slvaddr.text);

  ierr := ls_readi2c(adr,pBuf,wLength,ucstate,1000);

  if ierr = RET_OK then
  begin
    lb_i2cstat.Caption := ls_geti2cstring(ucstate);
    stri := '';
    if wLength > 0 then
    begin
      for i := 0 to wLength -1 do
        stri := stri + '0x'+ IntToHex(buf[i],2)+' ';
      i2cmemo.Lines.Add(stri);
    end;
  end;
  to_statusbar('I2C READ: '+ls_geterrorstring(ierr));
end;

procedure TForm1.btn_i2cwriteClick(Sender: TObject);
var buf : array[0..63] of byte;
    pBuf : Pointer;
    wLength, i, k : WORD;
    ierr : integer;
    adr : byte;
    ucstate : byte = 0;
    sLen : Word;
    sData : string;
begin
  sData := ed_i2cdata.text;
  sLen := length(sData);
  k := 0;
  i := 1;
  While i <= sLen - 1 do
  begin
    Buf[k] := StrToInt('$'+Copy(sData,i,2));
    i := i + 2;
    inc(k);
  end;

  pBuf := @buf[0];
  wLength := k;
  adr := strtoint('$'+ed_slvaddr.text);

  if wLength = 0 then
  begin
    to_statusbar('ERROR - INVALID I2C LENGTH');
    Exit;
  end;
  ierr := ls_writei2c(adr,pBuf,wLength,ucstate,1000);

  lb_i2cstat.Caption := ls_geti2cstring(ucstate);
  to_statusbar('I2C WRITE: '+ls_geterrorstring(ierr));
end;

procedure TForm1.btn_infoClick(Sender: TObject);
begin
  fminfo.devcount := cb_devices.Items.Count;
  fminfo.devindex := cb_devices.ItemIndex;
  fminfo.showmodal;
end;

procedure TForm1.btn_geti2cstatClick(Sender: TObject);
var stat : Byte = 0;
    ierr : integer;
begin
  ierr := ls_geti2cstat(stat,1000);
  if ierr = RET_OK then lb_i2cstat.Caption := ls_geti2cstring(stat)
  else lb_i2cstat.Caption := '???';
  to_statusbar('GET I2C STAT: '+ls_geterrorstring(ierr));
end;

procedure TForm1.btn_getsclClick(Sender: TObject);
var ierr : integer;
    sclfreq : Byte = 0;
begin
  ierr := ls_geti2cfreq(sclfreq,1000);
  if ierr = RET_OK then cb_i2cfreq.ItemIndex := sclfreq;
  to_statusbar('GET SCL FREQ: '+ls_geterrorstring(ierr));
end;

procedure TForm1.btn_setsclClick(Sender: TObject);
var ierr : integer;
begin
  ierr := ls_seti2cfreq(cb_i2cfreq.ItemIndex,1000);
  to_statusbar('SET SCL FREQ: '+ls_geterrorstring(ierr));
end;

end.

